from regression_utilities import *
import numpy as np
import hickle
from ase.io import read
from feat_settings import *
import pickle

frames = read(frames_file, feat_slice)
energy = []
for f in frames:
    energy.append(f.info["energy"])


def normalize(feats):
    """assumes input features in form (nstrc, featsize) already - INCLUDE commented out stuff if thats not the case"""
    fv = np.asarray(feats)
    nat = fv.shape[0]
    nfeat = fv.shape[1]
    fv = fv.reshape(nat * nfeat)  # , -1)
    nr = np.sqrt((fv**2).mean())  # nr = np.sqrt((fv**2).sum(axis=1).mean())
    fv = fv.reshape(nat, nfeat)
    fv /= nr
    return fv


energy = np.array(energy)
paths = [
    "n12l2rc3",
    "n12l2rc4",
    "n12l2rc8",
    "n12l2rc5",
    "n12l2rc10",
    "n12l2rc6",
    "n12l2rc12",
    "n15l2rc10",
    "n15l2rc12",
    "n15l2rc6",
]
pref = [
    "n12l2rc3-noRS",
    "n12l2rc4-noRS",
    "n12l2rc8-noRS",
    "n12l2rc5-noRS",
    "n12l2rc6-noRS",
    "n15l2rc10-noRS",
    "n15l2rc12-noRS",
]
for ip in range(len(paths)):
    print(paths[ip])
    feat_nu1 = np.array(
        pickle.load(open(paths[ip] + "/" + pref[ip] + "rho1i-feat.pickle", "rb"))
    )
    feat_nu2 = np.array(
        pickle.load(open(paths[ip] + "/" + pref[ip] + "rho2i-feat.pickle", "rb"))
    )
    feat_nu3 = np.array(
        pickle.load(open(paths[ip] + "/" + pref[ip] + "rho3i-feat.pickle", "rb"))
    )
    feat_nu11 = np.array(
        pickle.load(open(paths[ip] + "/" + pref[ip] + "rho11MPi-feat.pickle", "rb"))
    )
    maxtf = 0.9
    cv = 4
    alphas = np.geomspace(1e1, 1e-25, 60)
    trainfrac = [0.01, 0.025, 0.05, 0.1, 0.2, 0.4, 0.5, 0.75, maxtf]
    from numpy.random import shuffle

    seed = 2244
    np.random.seed(seed)
    # maxtrain = list(range(int(maxtf*len(frames))))
    maxtrain = list(range(len(frames)))
    shuffle(maxtrain)
    itest = maxtrain[int(maxtf * len(frames)) :]
    for itf, tf in enumerate(trainfrac[:]):
        itrain = maxtrain[: int(tf * len(frames))]
        print("\n nu1 regression with feature shape", feat_nu1.shape)
        ptrain_nu1, ptest_nu1 = test_regress_cv(
            feat_nu1, energy, itrain, itest, alphas, maxtf, tf, cv
        )
        rmse_nu1 = np.sqrt(np.mean((ptest_nu1 - energy[itest]) ** 2)) / np.std(
            energy[itest]
        )
        print("\n nu1  % RMSE ", rmse_nu1)

        print("\n nu2 regression with feature shape", feat_nu2.shape)
        ptrain_ps, ptest_ps = test_regress_cv(
            feat_nu2, energy, itrain, itest, alphas, maxtf, tf, cv
        )
        rmse_nu2 = np.sqrt(np.mean((ptest_ps - energy[itest]) ** 2)) / np.std(
            energy[itest]
        )
        print("\n PS  % RMSE ", rmse_nu2)

        print("\n nu3 regression with feature shape", feat_nu3.shape)
        ptrain_bis, ptest_bis = test_regress_cv(
            feat_nu3, energy, itrain, itest, alphas, maxtf, tf, cv
        )
        rmse_nu3 = np.sqrt(np.mean((ptest_bis - energy[itest]) ** 2)) / np.std(
            energy[itest]
        )
        print("\n Bispectrum % RMSE ", rmse_nu3)

        print("\n MP 1<-1 regression with feature shape", feat_nu11.shape)
        ptrain_mp, ptest_mp = test_regress_cv(
            feat_nu11, energy, itrain, itest, alphas, maxtf, tf, cv
        )
        rmse_MP11 = np.sqrt(np.mean((ptest_mp - energy[itest]) ** 2)) / np.std(
            energy[itest]
        )
        print("\n MP11  % RMSE ", rmse_MP11)

        np.save(
            "lc-%08d-%4.3f-%s.npy" % (seed, tf, pref[ip]),
            dict(
                seed=seed,
                frac=tf,
                itrain=itrain,
                itest=itest,
                pred_nu1=ptest_nu1,
                pred_nu1_train=ptrain_nu1,
                pred_nu3=ptest_bis,
                pred_nu3_train=ptrain_bis,
                pred_nu2=ptest_ps,
                pred_nu2_train=ptrain_ps,
                pred_mp=ptest_mp,
                pred_mp_train=ptrain_mp,
            ),
        )
